@@ -1,63 +1,20 @@
-/*
- * $Header: /home/cvspublic/jakarta-commons/digester/src/java/org/apache/commons/digester/CallMethodRule.java,v 1.23 2003/04/24 09:30:24 rdonkin Exp $
- * $Revision: 1.23 $
- * $Date: 2003/04/24 09:30:24 $
+/* $Id: CallMethodRule.java 499574 2007-01-24 21:25:51Z nuttycom $
  *
- * ====================================================================
- *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester;
@@ -69,16 +26,16 @@ import org.xml.sax.Attributes;
 
 
 /**
- * <p>Rule implementation that calls a method on the top (parent)
- * object, passing arguments collected from subsequent
- * <code>CallParamRule</code> rules or from the body of this
- * element. 
- * By using {@link #CallMethodRule(String methodName)} 
+ * <p>Rule implementation that calls a method on an object on the stack
+ * (normally the top/parent object), passing arguments collected from 
+ * subsequent <code>CallParamRule</code> rules or from the body of this
+ * element. </p>
+ *
+ * <p>By using {@link #CallMethodRule(String methodName)} 
  * a method call can be made to a method which accepts no
  * arguments.</p>
  *
- * <p>
- * Incompatible method parameter types are converted 
+ * <p>Incompatible method parameter types are converted 
  * using <code>org.apache.commons.beanutils.ConvertUtils</code>.
  * </p>
  *
@@ -86,22 +43,59 @@ import org.xml.sax.Attributes;
  * This increases the kinds of methods successfully and allows primitives
  * to be matched by passing in wrapper classes.
  * There are rare cases when {@link MethodUtils#invokeExactMethod} 
- * (the old defualt) is required.
+ * (the old default) is required.
  * This method is much stricter in it's reflection.
  * Setting the <code>UseExactMatch</code> to true reverts to the use of this 
  * method.</p>
  *
- * @author Craig McClanahan
- * @author Scott Sanders
- * @version $Revision: 1.23 $ $Date: 2003/04/24 09:30:24 $
+ * <p>Note that the target method is invoked when the  <i>end</i> of
+ * the tag the CallMethodRule fired on is encountered, <i>not</i> when the
+ * last parameter becomes available. This implies that rules which fire on
+ * tags nested within the one associated with the CallMethodRule will 
+ * fire before the CallMethodRule invokes the target method. This behaviour is
+ * not configurable. </p>
+ *
+ * <p>Note also that if a CallMethodRule is expecting exactly one parameter
+ * and that parameter is not available (eg CallParamRule is used with an
+ * attribute name but the attribute does not exist) then the method will
+ * not be invoked. If a CallMethodRule is expecting more than one parameter,
+ * then it is always invoked, regardless of whether the parameters were
+ * available or not; missing parameters are converted to the appropriate target
+ * type by calling ConvertUtils.convert. Note that the default ConvertUtils
+ * converters for the String type returns a null when passed a null, meaning
+ * that CallMethodRule will passed null for all String parameters for which
+ * there is no parameter info available from the XML. However parameters of
+ * type Float and Integer will be passed a real object containing a zero value
+ * as that is the output of the default ConvertUtils converters for those
+ * types when passed a null. You can register custom converters to change
+ * this behaviour; see the beautils library documentation for more info.</p>
+ *
+ * <p>Note that when a constructor is used with paramCount=0, indicating that
+ * the body of the element is to be passed to the target method, an empty 
+ * element will cause an <i>empty string</i> to be passed to the target method,
+ * not null. And if automatic type conversion is being applied (ie if the 
+ * target function takes something other than a string as a parameter) then 
+ * the conversion will fail if the converter class does not accept an empty 
+ * string as valid input.</p>
+ * 
+ * <p>CallMethodRule has a design flaw which can cause it to fail under
+ * certain rule configurations. All CallMethodRule instances share a single
+ * parameter stack, and all CallParamRule instances simply store their data
+ * into the parameter-info structure that is on the top of the stack. This
+ * means that two CallMethodRule instances cannot be associated with the
+ * same pattern without getting scrambled parameter data. This same issue
+ * also applies when a CallMethodRule matches some element X, a different 
+ * CallMethodRule matches a child element Y and some of the CallParamRules 
+ * associated with the first CallMethodRule match element Y or one of its 
+ * child elements. This issue has been present since the very first release
+ * of Digester. Note, however, that this configuration of CallMethodRule
+ * instances is not commonly required.</p>
  */
 
 public class CallMethodRule extends Rule {
 
-
     // ----------------------------------------------------------- Constructors
 
-
     /**
      * Construct a "call method" rule with the specified method name.  The
      * parameter types (if any) default to java.lang.String.
@@ -179,7 +173,26 @@ public class CallMethodRule extends Rule {
      */
     public CallMethodRule(String methodName,
                           int paramCount) {
+        this(0, methodName, paramCount);
+    }
 
+    /**
+     * Construct a "call method" rule with the specified method name.  The
+     * parameter types (if any) default to java.lang.String.
+     *
+     * @param targetOffset location of the target object. Positive numbers are
+     * relative to the top of the digester object stack. Negative numbers 
+     * are relative to the bottom of the stack. Zero implies the top
+     * object on the stack.
+     * @param methodName Method name of the parent method to call
+     * @param paramCount The number of parameters to collect, or
+     *  zero for a single argument from the body of this element.
+     */
+    public CallMethodRule(int targetOffset,
+                          String methodName,
+                          int paramCount) {
+
+        this.targetOffset = targetOffset;
         this.methodName = methodName;
         this.paramCount = paramCount;        
         if (paramCount == 0) {
@@ -201,7 +214,24 @@ public class CallMethodRule extends Rule {
      */
     public CallMethodRule(String methodName) {
     
-        this(methodName, 0, (Class[]) null);
+        this(0, methodName, 0, (Class[]) null);
+    
+    }
+    
+
+    /**
+     * Construct a "call method" rule with the specified method name.  
+     * The method should accept no parameters.
+     *
+     * @param targetOffset location of the target object. Positive numbers are
+     * relative to the top of the digester object stack. Negative numbers 
+     * are relative to the bottom of the stack. Zero implies the top
+     * object on the stack.
+     * @param methodName Method name of the parent method to call
+     */
+    public CallMethodRule(int targetOffset, String methodName) {
+    
+        this(targetOffset, methodName, 0, (Class[]) null);
     
     }
     
@@ -225,13 +255,40 @@ public class CallMethodRule extends Rule {
                             String methodName,
                             int paramCount, 
                             String paramTypes[]) {
+        this(0, methodName, paramCount, paramTypes);
+    }
 
+    /**
+     * Construct a "call method" rule with the specified method name and
+     * parameter types. If <code>paramCount</code> is set to zero the rule
+     * will use the body of this element as the single argument of the
+     * method, unless <code>paramTypes</code> is null or empty, in this
+     * case the rule will call the specified method with no arguments.
+     *
+     * @param targetOffset location of the target object. Positive numbers are
+     * relative to the top of the digester object stack. Negative numbers 
+     * are relative to the bottom of the stack. Zero implies the top
+     * object on the stack.
+     * @param methodName Method name of the parent method to call
+     * @param paramCount The number of parameters to collect, or
+     *  zero for a single argument from the body of ths element
+     * @param paramTypes The Java class names of the arguments
+     *  (if you wish to use a primitive type, specify the corresonding
+     *  Java wrapper class instead, such as <code>java.lang.Boolean</code>
+     *  for a <code>boolean</code> parameter)
+     */
+    public CallMethodRule(  int targetOffset,
+                            String methodName,
+                            int paramCount, 
+                            String paramTypes[]) {
+
+        this.targetOffset = targetOffset;
         this.methodName = methodName;
         this.paramCount = paramCount;
         if (paramTypes == null) {
             this.paramTypes = new Class[paramCount];
             for (int i = 0; i < this.paramTypes.length; i++) {
-                this.paramTypes[i] = "abc".getClass();
+                this.paramTypes[i] = String.class;
             }
         } else {
             // copy the parameter class names into an array
@@ -265,13 +322,41 @@ public class CallMethodRule extends Rule {
                             String methodName,
                             int paramCount, 
                             Class paramTypes[]) {
+        this(0, methodName, paramCount, paramTypes);
+    }
 
+    /**
+     * Construct a "call method" rule with the specified method name and
+     * parameter types. If <code>paramCount</code> is set to zero the rule
+     * will use the body of this element as the single argument of the
+     * method, unless <code>paramTypes</code> is null or empty, in this
+     * case the rule will call the specified method with no arguments.
+     *
+     * @param targetOffset location of the target object. Positive numbers are
+     * relative to the top of the digester object stack. Negative numbers 
+     * are relative to the bottom of the stack. Zero implies the top
+     * object on the stack.
+     * @param methodName Method name of the parent method to call
+     * @param paramCount The number of parameters to collect, or
+     *  zero for a single argument from the body of ths element
+     * @param paramTypes The Java classes that represent the
+     *  parameter types of the method arguments
+     *  (if you wish to use a primitive type, specify the corresonding
+     *  Java wrapper class instead, such as <code>java.lang.Boolean.TYPE</code>
+     *  for a <code>boolean</code> parameter)
+     */
+    public CallMethodRule(  int targetOffset,
+                            String methodName,
+                            int paramCount, 
+                            Class paramTypes[]) {
+
+        this.targetOffset = targetOffset;
         this.methodName = methodName;
         this.paramCount = paramCount;
         if (paramTypes == null) {
             this.paramTypes = new Class[paramCount];
             for (int i = 0; i < this.paramTypes.length; i++) {
-                this.paramTypes[i] = "abc".getClass();
+                this.paramTypes[i] = String.class;
             }
         } else {
             this.paramTypes = new Class[paramTypes.length];
@@ -292,6 +377,13 @@ public class CallMethodRule extends Rule {
     protected String bodyText = null;
 
 
+    /** 
+     * location of the target object for the call, relative to the
+     * top of the digester object stack. The default value of zero
+     * means the target object is the one on top of the stack.
+     */
+    protected int targetOffset = 0;
+
     /**
      * The method name to call on the parent object.
      */
@@ -415,20 +507,27 @@ public class CallMethodRule extends Rule {
                 }
             }
             
-            // In the case where the parameter for the method
-            // is taken from an attribute, and that attribute
-            // isn't actually defined in the source XML file,
-            // skip the method call
+            // In the case where the target method takes a single parameter
+            // and that parameter does not exist (the CallParamRule never
+            // executed or the CallParamRule was intended to set the parameter
+            // from an attribute but the attribute wasn't present etc) then
+            // skip the method call.
+            //
+            // This is useful when a class has a "default" value that should
+            // only be overridden if data is present in the XML. I don't
+            // know why this should only apply to methods taking *one*
+            // parameter, but it always has been so we can't change it now.
             if (paramCount == 1 && parameters[0] == null) {
                 return;
             }
 
         } else if (paramTypes != null && paramTypes.length != 0) {
+            // Having paramCount == 0 and paramTypes.length == 1 indicates
+            // that we have the special case where the target method has one
+            // parameter being the body text of the current element.
 
-            // In the case where the parameter for the method
-            // is taken from the body text, but there is no
-            // body text included in the source XML file,
-            // skip the method call
+            // There is no body text included in the source XML file,
+            // so skip the method call
             if (bodyText == null) {
                 return;
             }
@@ -437,9 +536,14 @@ public class CallMethodRule extends Rule {
             parameters[0] = bodyText;
             if (paramTypes.length == 0) {
                 paramTypes = new Class[1];
-                paramTypes[0] = "abc".getClass();
+                paramTypes[0] = String.class;
             }
 
+        } else {
+            // When paramCount is zero and paramTypes.length is zero it
+            // means that we truly are calling a method with no parameters.
+            // Nothing special needs to be done here.
+            ;
         }
 
         // Construct the parameter values array we will need
@@ -450,8 +554,8 @@ public class CallMethodRule extends Rule {
             // convert nulls and convert stringy parameters 
             // for non-stringy param types
             if(
-                parameters[i] == null
-                || (parameters[i] instanceof String && 
+                parameters[i] == null ||
+                 (parameters[i] instanceof String && 
                    !String.class.isAssignableFrom(paramTypes[i]))) {
                 
                 paramValues[i] =
@@ -461,17 +565,33 @@ public class CallMethodRule extends Rule {
             }
         }
 
+        // Determine the target object for the method call
+        Object target;
+        if (targetOffset >= 0) {
+            target = digester.peek(targetOffset);
+        } else {
+            target = digester.peek( digester.getCount() + targetOffset );
+        }
+        
+        if (target == null) {
+            StringBuffer sb = new StringBuffer();
+            sb.append("[CallMethodRule]{");
+            sb.append(digester.match);
+            sb.append("} Call target is null (");
+            sb.append("targetOffset=");
+            sb.append(targetOffset);
+            sb.append(",stackdepth=");
+            sb.append(digester.getCount());
+            sb.append(")");
+            throw new org.xml.sax.SAXException(sb.toString());
+        }
+        
         // Invoke the required method on the top object
-        Object top = digester.peek();
         if (digester.log.isDebugEnabled()) {
             StringBuffer sb = new StringBuffer("[CallMethodRule]{");
             sb.append(digester.match);
             sb.append("} Call ");
-            if (top == null) {
-                sb.append("[NULL TOP]");
-            } else {
-                sb.append(top.getClass().getName());
-            }
+            sb.append(target.getClass().getName());
             sb.append(".");
             sb.append(methodName);
             sb.append("(");
@@ -495,16 +615,19 @@ public class CallMethodRule extends Rule {
             digester.log.debug(sb.toString());
         }
         
+        Object result = null;
         if (useExactMatch) {
             // invoke using exact match
-            MethodUtils.invokeExactMethod(top, methodName,
+            result = MethodUtils.invokeExactMethod(target, methodName,
                 paramValues, paramTypes);
                 
         } else {
             // invoke using fuzzier match
-            MethodUtils.invokeMethod(top, methodName,
+            result = MethodUtils.invokeMethod(target, methodName,
                 paramValues, paramTypes);            
         }
+        
+        processMethodCallResult(result);
     }
 
 
@@ -517,6 +640,15 @@ public class CallMethodRule extends Rule {
 
     }
 
+    /**
+     * Subclasses may override this method to perform additional processing of the 
+     * invoked method's result.
+     *
+     * @param result the Object returned by the method invoked, possibly null
+     */
+    protected void processMethodCallResult(Object result) {
+        // do nothing
+    }
 
     /**
      * Render a printable version of this Rule.
