@@ -1,62 +1,20 @@
-/*
- * $Header: /home/cvspublic/jakarta-commons/digester/src/java/org/apache/commons/digester/ExtendedBaseRules.java,v 1.6 2003/04/16 11:23:50 jstrachan Exp $
- * $Revision: 1.6 $
- * $Date: 2003/04/16 11:23:50 $
- * ====================================================================
+/* $Id: ExtendedBaseRules.java 471661 2006-11-06 08:09:25Z skitching $
  *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester;
@@ -82,96 +40,130 @@ import java.util.Map;
  *
  * <h4>When should you use this rather than the original?</h4>
  *
- * <p>These rules are complex and slower but offer more functionality.
- * The <code>RulesBase</code> matching set allows interaction between patterns.
- * This allows sophisticated matching schema to be created
- * but it also means that it can be hard to create and debug mappings
- * for complex schema.
- * This extension introduces <em>universal</em> versions of these patterns
- * that always act independently.</p>
+ * <p>
+ * This pattern-matching engine is complex and slower than the basic
+ * default RulesBase class, but offers more functionality:
+ * <ul>
+ * <li>Universal patterns allow patterns to be specified which will match
+ *  regardless of whether there are "better matching" patterns available.</li>
+ * <li>Parent-match patterns (eg "a/b/?") allow matching for all direct
+ *  children of a specified element.</li>
+ * <li>Ancestor-match patterns (eg "a/b/*") allow matching all elements
+ *  nested within a specified element to any nesting depth.</li>
+ * <li>Completely-wild patterns ("*" or "!*") allow matching all elements.</li>
+ * </ul>
+ * </p>
  *
- * <p>Another three kinds of matching pattern are also introduced.
- * The parent matchs allow common method to be easily called for children.
- * The wildcard match allows rules to be specified for all elements.</p>
+ * <h4>Universal Match Patterns</h4>
+ * 
+ * <p>The default RulesBase pattern-matching engine always attempts to find
+ * the "best matching pattern", and will ignore rules associated with other
+ * patterns that match but are not "as good". As an example, if the pattern
+ * "a/b/c" is associated with rules 1 and 2, and "*&#47;c" is associated with
+ * rules 3 and 4 then element "a/b/c" will cause only rules 1 and 2 to execute.
+ * Rules 3 and 4 do have matching patterns, but because the patterns are shorter
+ * and include wildcard characters they are regarded as being "not as good" as
+ * a direct match. In general, exact patterns are better than wildcard patterns,
+ * and among multiple patterns with wildcards, the longest is preferred.
+ * See the RulesBase class for more information.</p>
  *
- * <h4>The additional matching patterns:</h4>
+ * <p>This feature of preferring "better" patterns can be a powerful tool.
+ * However it also means that patterns can interact in unexpected ways.</p>
+ *
+ * <p>When using the ExtendedBaseRules, any pattern prefixed with '!' bypasses 
+ * the "best match" feature. Even if there is an exact match or a longer 
+ * wildcard match, patterns prefixed by '!' will still be tested to see if 
+ * they match, and if so their associated Rule objects will be included in
+ * the set of rules to be executed in the normal manner.</p>
  *
  * <ul>
- * <li><em>Parent Match </em> - Will match child elements of a particular
- *     kind of parent.  This is useful if a parent has a particular method
- *     to call.
- *     <ul>
- *     <li><code>"a/b/c/?"</code> matches any child whose parent matches
- *         <code>"a/b/c"</code>.  Exact parent rules take precedence over
- *         standard wildcard tail endings.</li>
- *     <li><code>"*&#47;a/b/c/?"</code> matches any child whose parent matches
- *         "*&#47;a/b/c"</code>.  The longest matching still applies to parent
- *         matches but the length excludes the '?', which effectively means
- *         that standard wildcard matches with the same level of depth are
- *         chosen in preference.</li>
- *     </ul></li>
- * <li><em>Ancester Match</em> - Will match elements who parentage includes 
- * 	a particular sequence of elements.
- * 	<ul>
- *      <li><code>"a/b/*"</code> matches any element whose parentage path starts with
- *	    'a' then 'b'. Exact parent and parent match rules take precedence. The longest
- *           ancester match will take precedence.</li>
- *	<li><code>"*&#47;a/b/*"</code> matches any elements whose parentage path contains
- *           an element 'a' followed by an element 'b'. The longest matching still applies
- *           but the length excludes the '*' at the end.</li>
- *      </ul>
- * </li>
- * <li><em>Universal Wildcard Match</em> -  Any pattern prefixed with '!'
- *     bypasses the longest matching rule.  Even if there is an exact match
- *     or a longer wildcard match,  patterns prefixed by '!' will still be
- *     tested to see if they match.  This can be used for example to specify
- *     universal construction rules.
- *     <ul>
- *     <li>Pattern <code>"!*&#47;a/b"</code> matches whenever an 'b' element
- *         is inside an 'a'.</li>
- *     <li>Pattern <code>"!a/b/?"</code> matches any child of a parent
- *         matching <code>"a/b"</code>.</li>
- *     <li>Pattern <code>"!*&#47;a/b/?"</code> matches any child of a parent
- *         matching <code>"!*&#47;a/b"</code></li>
- *     <li>Pattern <code>"!a/b/*"</code> matches any element whose parentage path starts with
- *	    "a" then "b".</li>
- *     <li>Pattern <code>"!*&#47;a/b/*"</code> matches any elements whose parentage path contains
- *          'a/b'</li>
- *    </ul></li>
- * <li><em>Wild Match</em>
- *     <ul>
- *     <li>Pattern <code>"*"</code> matches every pattern that isn't matched
- *         by any other basic rule.</li>
- *     <li>Pattern <code>"!*"</code> matches every pattern.</li>
- *     </ul></li>
+ * <li>Pattern <code>"!*&#47;a/b"</code> matches whenever an 'b' element
+ *     is inside an 'a'.</li>
+ * <li>Pattern <code>"!a/b/?"</code> matches any child of a parent
+ *     matching <code>"a/b"</code> (see "Parent Match Patterns").</li>
+ * <li>Pattern <code>"!*&#47;a/b/?"</code> matches any child of a parent
+ *     matching <code>"!*&#47;a/b"</code> (see "Parent Match Patterns").</li>
+ * <li>Pattern <code>"!a/b/*"</code> matches any element whose path
+ *     starts with "a" then "b" (see "Ancestor Match Patterns").</li>
+ * <li>Pattern <code>"!*&#47;a/b/*"</code> matches any elements whose path
+ *     contains 'a/b' (see "Ancestor Match Patterns").</li>
  * </ul>
  *
+ * <h4>Parent Match Patterns</h4>
+ *
+ * <p>
+ * These will match direct child elements of a particular parent element.
+ * <ul>
+ * <li>
+ *  <code>"a/b/c/?"</code> matches any child whose parent matches
+ *  <code>"a/b/c"</code>.  Exact parent rules take precedence over Ancestor
+ *  Match patterns.
+ * </li>
+ * <li>
+ *  <code>"*&#47;a/b/c/?"</code> matches any child whose parent matches
+ *  <code>"*&#47;a/b/c"</code>.  The longest matching still applies to parent
+ *  matches but the length excludes the '?', which effectively means
+ *  that standard wildcard matches with the same level of depth are
+ *  chosen in preference.
+ * </li>
+ * </ul>
+ * </p>
+ *
+ * <h4>Ancestor Match Patterns</h4>
+ *
+ * <p>
+ * These will match elements whose parentage includes a particular sequence
+ * of elements.
+ * <ul>
+ * <li>
+ *  <code>"a/b/*"</code> matches any element whose path starts with
+ *  'a' then 'b'. Exact parent and parent match rules take precedence. 
+ *  The longest ancestor match will take precedence.
+ * </li>
+ * <li>
+ *  <code>"*&#47;a/b/*"</code> matches any elements whose path contains
+ *  an element 'a' followed by an element 'b'. The longest matching still 
+ *  applies but the length excludes the '*' at the end.
+ * </li>
+ * </ul>
+ * </p>
+ *
+ * <h4>Completely Wild Patterns</h4>
+ *
+ * <p>Pattern <code>"*"</code> matches every pattern that isn't matched by
+ * any other basic rule.</p>
+ *
+ * <p>Pattern <code>"!*"</code> matches every pattern.</p>
+ *
  * <h4>Using The Extended Rules</h4>
  *
- * <p>The most important thing to remember
- * when using the extended rules is that universal
- * and non-universal patterns are completely independent.
- * Universal patterns are never effected by the addition of new patterns
- * or the removal of existing ones.
- * Non-universal patterns are never effected
- * by the addition of new <em>universal</em> patterns
- * or the removal of existing <em>universal</em> patterns.
- * As in the basic matching rules, non-universal (basic) patterns
- * <strong>can</strong> be effected
- * by the addition of new <em>non-universal</em> patterns
- * or the removal of existing <em>non-universal</em> patterns.
- * <p> This means that you can use universal patterns
- * to build up the simple parts of your structure
- * - for example defining universal creation and property setting rules.
- * More sophisticated and complex mapping will require non-universal patterns
- * and this might mean that some of the universal rules will need to be
- * replaced by a series of
- * special cases using non-universal rules.
- * But by using universal rules as your backbone,
- * these additions should not break your existing rules.</p>
+ * <p>By default, a Digester instance uses a {@link RulesBase} instance as
+ * its pattern matching engine. To use an ExtendedBaseRules instance, call
+ * the Digester.setRules method before adding any Rule objects to the digester
+ * instance:
+ * <pre>
+ *     Digester digester = new Digester();
+ *     digester.setRules( new ExtendedBaseRules() );
+ * </pre></p>
  *
- * @author Robert Burrell Donkin <robertdonkin@mac.com>
- * @version $Revision: 1.6 $ $Date: 2003/04/16 11:23:50 $
+ * <p>The most important thing to remember when using the extended rules is 
+ * that universal and non-universal patterns are completely independent.
+ * Universal patterns are never affected by the addition of new patterns
+ * or the removal of existing ones. Non-universal patterns are never affected
+ * by the addition of new <em>universal</em> patterns or the removal of 
+ * existing <em>universal</em> patterns. As in the basic matching rules, 
+ * non-universal (basic) patterns <strong>can</strong> be affected by the 
+ * addition of new <em>non-universal</em> patterns or the removal of existing 
+ * <em>non-universal</em> patterns, because only rules associated with the
+ * "best matching" pattern for each xml element are executed.
+ *
+ * <p> This means that you can use universal patterns to build up the simple 
+ * parts of your structure - for example defining universal creation and 
+ * property setting rules. More sophisticated and complex mapping will require 
+ * non-universal patterns and this might mean that some of the universal rules 
+ * will need to be replaced by a series of special cases using non-universal 
+ * rules. But by using universal rules as your backbone, these additions 
+ * should not break your existing rules.</p>
  */
 
 
@@ -284,7 +276,7 @@ public class ExtendedBaseRules extends RulesBase {
                 } else {
                     // we don't have a match yet - so try exact ancester
                     //
-                    rulesList = findExactAncesterMatch(parentPattern);
+                    rulesList = findExactAncesterMatch(pattern);
                     if (rulesList != null) {
                         // we have a match!
                         // so ignore all basic matches from now on
@@ -301,6 +293,8 @@ public class ExtendedBaseRules extends RulesBase {
 
         // Find the longest key, ie more discriminant
         String longKey = "";
+        int longKeyLength = 0;
+        
         Iterator keys = this.cache.keySet().iterator();
         while (keys.hasNext()) {
             String key = (String) keys.next();
@@ -315,24 +309,42 @@ public class ExtendedBaseRules extends RulesBase {
 
                     
             // don't need to check exact matches
-            if (key.startsWith("*/") || (isUniversal && key.endsWith("/*"))) {
+            boolean wildcardMatchStart = key.startsWith("*/");
+            boolean wildcardMatchEnd = key.endsWith("/*");
+            if (wildcardMatchStart || (isUniversal && wildcardMatchEnd)) {
 
                 boolean parentMatched = false;
                 boolean basicMatched = false;
                 boolean ancesterMatched = false;
-                if (key.endsWith("/?")) {
+                
+                boolean parentMatchEnd = key.endsWith("/?");
+                if (parentMatchEnd) {
                     // try for a parent match
                     parentMatched = parentMatch(key, pattern, parentPattern);
 
-                } else if (key.endsWith("/*")) {
+                } else if (wildcardMatchEnd) {
                     // check for ancester match
-                    int patternStart = 0;
-                    if (key.startsWith("*/")) {
-                        patternStart = 2;
-                    }
-                    ancesterMatched = 
-                        (pattern.lastIndexOf(key.substring(patternStart, key.length() - 2)) > -1);
-                                    
+                    if (wildcardMatchStart) {
+                        String patternBody = key.substring(2, key.length() - 2);
+                        if (pattern.endsWith(patternBody)) {
+                            ancesterMatched = true;
+                        } else {
+                            ancesterMatched = (pattern.indexOf(patternBody + "/") > -1);
+                        }
+                    } else {
+                        String bodyPattern = key.substring(0, key.length() - 2);
+                        if (pattern.startsWith(bodyPattern))
+                        {
+                            if (pattern.length() == bodyPattern.length()) {
+                                // exact match
+                                ancesterMatched = true;
+                            } else {
+                                ancesterMatched = ( pattern.charAt(bodyPattern.length()) == '/' );
+                            }
+                        } else {
+                            ancesterMatched = false;  
+                        } 
+                    }               
                 } else {
                     // try for a base match
                     basicMatched = basicMatch(key, pattern);
@@ -350,16 +362,25 @@ public class ExtendedBaseRules extends RulesBase {
                     } else {
                         if (!ignoreBasicMatches) {
                             // ensure that all parent matches are SHORTER
-                            // than rules with same level of matching
+                            // than rules with same level of matching.
+                            //
+                            // the calculations below don't work for universal
+                            // matching, but we don't care because in that case
+                            // this if-stmt is not entered.
                             int keyLength = key.length();
-                            if (parentMatched || ancesterMatched) {
-                                keyLength--;
+                            if (wildcardMatchStart) {
+                                --keyLength;
+                            }
+                            if (wildcardMatchEnd) {
+                                --keyLength;
+                            } else if (parentMatchEnd) {
+                                --keyLength;
                             }
 
-
-                            if (keyLength > longKey.length()) {
+                            if (keyLength > longKeyLength) {
                                 rulesList = (List) this.cache.get(key);
                                 longKey = key;
+                                longKeyLength = keyLength;
                             }
                         }
                     }
