@@ -1,59 +1,20 @@
-/*
- * ====================================================================
+/* $Id: NodeCreateRule.java 730333 2008-12-31 07:20:47Z rahul $
  *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights 
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:  
- *       "This product includes software developed by the 
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written 
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester;
@@ -71,7 +32,6 @@ import org.w3c.dom.Node;
 import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.SAXException;
-import org.xml.sax.XMLReader;
 import org.xml.sax.helpers.DefaultHandler;
 
 
@@ -109,9 +69,7 @@ import org.xml.sax.helpers.DefaultHandler;
  * <p><strong>Note</strong> that the current implementation does not set the namespace prefixes
  * in the exported nodes. The (usually more important) namespace URIs are set,
  * of course.</p>
- * 
- * @author Christopher Lenz
- * @version $Revision: 1.2 $ $Date: 2003/02/02 16:09:53 $
+ *
  * @since Digester 1.4
  */
 
@@ -124,8 +82,6 @@ public class NodeCreateRule extends Rule {
     /**
      * The SAX content handler that does all the actual work of assembling the 
      * DOM node tree from the SAX events.
-     * 
-     * @author <a href="mailto:cmlenz@apache.org">Christopher Lenz</a>
      */
     private class NodeBuilder
         extends DefaultHandler {
@@ -155,7 +111,7 @@ public class NodeCreateRule extends Rule {
             this.root = root;
             this.top = root;
             
-            oldContentHandler = digester.getXMLReader().getContentHandler();
+            oldContentHandler = digester.getCustomContentHandler();
 
         }
 
@@ -194,12 +150,56 @@ public class NodeCreateRule extends Rule {
          */
         protected Node top = null;
 
+        /**
+         * The text content of the current top DOM node.
+         */
+        protected StringBuffer topText = new StringBuffer();
+
+
+        // --------------------------------------------- Helper Methods
+
+        /**
+         * Appends a {@link org.w3c.dom.Text Text} node to the current node
+         * if the content reported by the parser is not purely whitespace.
+         */
+        private void addTextIfPresent() throws SAXException {
+            if (topText.length() > 0) {
+                String str = topText.toString();
+                topText.setLength(0);
+
+                if (str.trim().length() > 0) {
+                    // The contained text is not *pure* whitespace, so create
+                    // a text node to hold it. Note that the "untrimmed" text
+                    // is stored in the node.
+                    try {
+                        top.appendChild(doc.createTextNode(str));
+                    } catch (DOMException e) {
+                        throw new SAXException(e.getMessage());
+                    }
+                }
+            }
+        }
 
         // --------------------------------------------- ContentHandler Methods
 
 
         /**
-         * Appends a {@link org.w3c.dom.Text Text} node to the current node.
+         * Handle notification about text embedded within the current node.
+         * <p>
+         * An xml parser calls this when text is found. We need to ensure that this
+         * text gets attached to the new Node we are creating - except in the case
+         * where the only text in the node is whitespace.
+         * <p>
+         * There is a catch, however. According to the sax specification, a parser
+         * does not need to pass all of the text content of a node in one go; it can
+         * make multiple calls passing part of the data on each call. In particular,
+         * when the body of an element includes xml entity-references, at least some
+         * parsers make a separate call to this method to pass just the entity content.
+         * <p>
+         * In this method, we therefore just append the provided text to a
+         * "current text" buffer. When the element end is found, or a child element
+         * is found then we can check whether we have all-whitespace. See method
+         * addTextIfPresent. 
          * 
          * @param ch the characters from the XML document
          * @param start the start position in the array
@@ -209,15 +209,7 @@ public class NodeCreateRule extends Rule {
         public void characters(char[] ch, int start, int length)
             throws SAXException {
 
-            try {
-                String str = new String(ch, start, length);
-                if (str.trim().length() > 0) { 
-                    top.appendChild(doc.createTextNode(str));
-                }
-            } catch (DOMException e) {
-                throw new SAXException(e.getMessage());
-            }
-
+            topText.append(ch, start, length);
         }
 
 
@@ -233,10 +225,11 @@ public class NodeCreateRule extends Rule {
                                String qName)
             throws SAXException {
             
+            addTextIfPresent();
+
             try {
                 if (depth == 0) {
-                    getDigester().getXMLReader().setContentHandler(
-                        oldContentHandler);
+                    getDigester().setCustomContentHandler(oldContentHandler);
                     getDigester().push(root);
                     getDigester().endElement(namespaceURI, localName, qName);
                 }
@@ -286,6 +279,8 @@ public class NodeCreateRule extends Rule {
                                  String qName, Attributes atts)
             throws SAXException {
 
+            addTextIfPresent();
+
             try {
                 Node previousTop = top;
                 if ((localName == null) || (localName.length() == 0)) { 
@@ -416,8 +411,12 @@ public class NodeCreateRule extends Rule {
 
 
     /**
-     * Implemented to replace the content handler currently in use by a 
-     * {@link NodeBuilder NodeCreateRule.NodeBuilder}.
+     * When this method fires, the digester is told to forward all SAX
+     * ContentHandler events to the builder object, resulting in a DOM being
+     * built instead of normal digester rule-handling occurring. When the
+     * end of the current xml element is encountered, the original content 
+     * handler is restored (expected to be NULL, allowing normal Digester
+     * operations to continue).
      * 
      * @param namespaceURI the namespace URI of the matching element, or an 
      *   empty string if the parser is not namespace aware or the element has
@@ -430,7 +429,6 @@ public class NodeCreateRule extends Rule {
     public void begin(String namespaceURI, String name, Attributes attributes)
         throws Exception {
 
-        XMLReader xmlReader = getDigester().getXMLReader();
         Document doc = documentBuilder.newDocument();
         NodeBuilder builder = null;
         if (nodeType == Node.ELEMENT_NODE) {
@@ -440,7 +438,7 @@ public class NodeCreateRule extends Rule {
                     doc.createElementNS(namespaceURI, name);
                 for (int i = 0; i < attributes.getLength(); i++) {
                     element.setAttributeNS(attributes.getURI(i),
-                                           attributes.getLocalName(i),
+                                           attributes.getQName(i),
                                            attributes.getValue(i));
                 }
             } else {
@@ -454,7 +452,20 @@ public class NodeCreateRule extends Rule {
         } else {
             builder = new NodeBuilder(doc, doc.createDocumentFragment());
         }
-        xmlReader.setContentHandler(builder);
+        // the NodeBuilder constructor has already saved the original
+        // value of the digester's custom content handler (expected to
+        // be null, but we save it just in case). So now we just
+        // need to tell the digester to forward events to the builder.
+        getDigester().setCustomContentHandler(builder);
+    }
+
+
+    /**
+     * Pop the Node off the top of the stack.
+     */
+    public void end() throws Exception {
+
+        digester.pop();
 
     }
 
