@@ -1,59 +1,20 @@
-/*
- * ====================================================================
+/* $Id: DigesterRuleParser.java 471661 2006-11-06 08:09:25Z skitching $
  *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester.xmlrules;
@@ -68,21 +29,30 @@ import java.util.List;
 import java.util.Set;
 import java.util.StringTokenizer;
 
+import org.apache.commons.beanutils.ConvertUtils;
+
 import org.apache.commons.collections.ArrayStack;
+
 import org.apache.commons.digester.AbstractObjectCreationFactory;
 import org.apache.commons.digester.BeanPropertySetterRule;
 import org.apache.commons.digester.CallMethodRule;
 import org.apache.commons.digester.CallParamRule;
 import org.apache.commons.digester.Digester;
 import org.apache.commons.digester.FactoryCreateRule;
+import org.apache.commons.digester.NodeCreateRule;
 import org.apache.commons.digester.ObjectCreateRule;
 import org.apache.commons.digester.Rule;
 import org.apache.commons.digester.RuleSetBase;
 import org.apache.commons.digester.Rules;
+import org.apache.commons.digester.SetNestedPropertiesRule;
 import org.apache.commons.digester.SetNextRule;
 import org.apache.commons.digester.SetPropertiesRule;
 import org.apache.commons.digester.SetPropertyRule;
+import org.apache.commons.digester.SetRootRule;
 import org.apache.commons.digester.SetTopRule;
+import org.apache.commons.digester.ObjectParamRule;
+
+import org.w3c.dom.Node;
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 
@@ -91,10 +61,7 @@ import org.xml.sax.SAXException;
  * This is a RuleSet that parses XML into Digester rules, and then
  * adds those rules to a 'target' Digester.
  *
- * @author David H. Martin - Initial Contribution
- * @author Scott Sanders   - Added ASL, removed external dependencies
- * @author Bradley M. Handy - Bean Property Setter Rule addition
- * 
+ * @since 1.2
  */
 
 public class DigesterRuleParser extends RuleSetBase {
@@ -111,7 +78,9 @@ public class DigesterRuleParser extends RuleSetBase {
      * from the Rules XML document.
      */
     protected Digester targetDigester;
-    
+
+    /** See {@link #setBasePath}. */
+    protected String basePath = "";
     
     /**
      * A stack whose toString method returns a '/'-separated concatenation
@@ -189,6 +158,24 @@ public class DigesterRuleParser extends RuleSetBase {
         targetDigester = d;
     }
     
+    /**
+     * Set a base pattern beneath which all the rules loaded by this
+     * object will be registered. If this string is not empty, and does
+     * not end in a "/", then one will be added.
+     *
+     * @since 1.6
+     */
+    public void setBasePath(String path) {
+        if (path == null) {
+            basePath = "";
+        }
+        else if ((path.length() > 0) && !path.endsWith("/")) {
+            basePath = path + "/";
+        } else {
+            basePath = path;
+        }
+    }
+
     /**
      * Sets the location of the digester rules DTD. This is the DTD used
      * to validate the rules XML file.
@@ -216,7 +203,8 @@ public class DigesterRuleParser extends RuleSetBase {
      * @param rule a Rule to add to the target digester.
      */
     public void add(Rule rule) {
-        targetDigester.addRule(patternStack.toString(), rule);
+        targetDigester.addRule(
+            basePath + patternStack.toString(), rule);
     }
     
     
@@ -244,6 +232,10 @@ public class DigesterRuleParser extends RuleSetBase {
         digester.addFactoryCreate("*/call-method-rule", new CallMethodRuleFactory());
         digester.addRule("*/call-method-rule", new PatternRule("pattern"));
         digester.addSetNext("*/call-method-rule", "add", ruleClassName);
+
+        digester.addFactoryCreate("*/object-param-rule", new ObjectParamRuleFactory());
+        digester.addRule("*/object-param-rule", new PatternRule("pattern"));
+        digester.addSetNext("*/object-param-rule", "add", ruleClassName);
         
         digester.addFactoryCreate("*/call-param-rule", new CallParamRuleFactory());
         digester.addRule("*/call-param-rule", new PatternRule("pattern"));
@@ -257,6 +249,10 @@ public class DigesterRuleParser extends RuleSetBase {
         digester.addRule("*/object-create-rule", new PatternRule("pattern"));
         digester.addSetNext("*/object-create-rule", "add", ruleClassName);
         
+        digester.addFactoryCreate("*/node-create-rule", new NodeCreateRuleFactory());
+        digester.addRule("*/node-create-rule", new PatternRule("pattern"));
+        digester.addSetNext("*/node-create-rule", "add", ruleClassName);
+        
         digester.addFactoryCreate("*/set-properties-rule", new SetPropertiesRuleFactory());
         digester.addRule("*/set-properties-rule", new PatternRule("pattern"));
         digester.addSetNext("*/set-properties-rule", "add", ruleClassName);
@@ -267,6 +263,12 @@ public class DigesterRuleParser extends RuleSetBase {
         digester.addRule("*/set-property-rule", new PatternRule("pattern"));
         digester.addSetNext("*/set-property-rule", "add", ruleClassName);
         
+        digester.addFactoryCreate("*/set-nested-properties-rule", new SetNestedPropertiesRuleFactory());
+        digester.addRule("*/set-nested-properties-rule", new PatternRule("pattern"));
+        digester.addSetNext("*/set-nested-properties-rule", "add", ruleClassName);
+        
+        digester.addRule("*/set-nested-properties-rule/alias", new SetNestedPropertiesAliasRule());
+        
         digester.addFactoryCreate("*/set-top-rule", new SetTopRuleFactory());
         digester.addRule("*/set-top-rule", new PatternRule("pattern"));
         digester.addSetNext("*/set-top-rule", "add", ruleClassName);
@@ -274,6 +276,9 @@ public class DigesterRuleParser extends RuleSetBase {
         digester.addFactoryCreate("*/set-next-rule", new SetNextRuleFactory());
         digester.addRule("*/set-next-rule", new PatternRule("pattern"));
         digester.addSetNext("*/set-next-rule", "add", ruleClassName);
+        digester.addFactoryCreate("*/set-root-rule", new SetRootRuleFactory());
+        digester.addRule("*/set-root-rule", new PatternRule("pattern"));
+        digester.addSetNext("*/set-root-rule", "add", ruleClassName);
     }
     
     
@@ -291,7 +296,6 @@ public class DigesterRuleParser extends RuleSetBase {
         private String pattern = null;
         
         /**
-         * @param digester the Digester used to parse the rules XML file
          * @param attrName The name of the attribute containing the pattern
          */
         public PatternRule(String attrName) {
@@ -363,7 +367,11 @@ public class DigesterRuleParser extends RuleSetBase {
          */
         private void includeXMLRules(String fileName)
                         throws IOException, SAXException, CircularIncludeException {
-            URL fileURL = DigesterRuleParser.this.getClass().getClassLoader().getResource(fileName);
+            ClassLoader cl = Thread.currentThread().getContextClassLoader();
+            if (cl == null) {
+                cl = DigesterRuleParser.this.getClass().getClassLoader();
+            }
+            URL fileURL = cl.getResource(fileName);
             if (fileURL == null) {
                 throw new FileNotFoundException("File \"" + fileName + "\" not found.");
             }
@@ -440,7 +448,13 @@ public class DigesterRuleParser extends RuleSetBase {
          * by concatenating the pattern prefix with the given pattern.
          */
         public void add(String pattern, Rule rule) {
-            delegate.add(prefix + pattern, rule);
+            StringBuffer buffer = new StringBuffer();
+            buffer.append(prefix);
+            if (!pattern.startsWith("/")) {
+                buffer.append('/'); 
+            }
+            buffer.append(pattern);
+            delegate.add(buffer.toString(), rule);
         }
         
         /**
@@ -533,31 +547,53 @@ public class DigesterRuleParser extends RuleSetBase {
         public Object createObject(Attributes attributes) {
             Rule callMethodRule = null;
             String methodName = attributes.getValue("methodname");
+
+            // Select which element is to be the target. Default to zero,
+            // ie the top object on the stack.
+            int targetOffset = 0;
+            String targetOffsetStr = attributes.getValue("targetoffset");
+            if (targetOffsetStr != null) {
+                targetOffset = Integer.parseInt(targetOffsetStr);
+            }
+
             if (attributes.getValue("paramcount") == null) {
                 // call against empty method
-                callMethodRule = new CallMethodRule(methodName);
+                callMethodRule = new CallMethodRule(targetOffset, methodName);
             
             } else {
                 int paramCount = Integer.parseInt(attributes.getValue("paramcount"));
                 
                 String paramTypesAttr = attributes.getValue("paramtypes");
                 if (paramTypesAttr == null || paramTypesAttr.length() == 0) {
-                    callMethodRule = new CallMethodRule(methodName, paramCount);
+                    callMethodRule = new CallMethodRule(targetOffset, methodName, paramCount);
                 } else {
-                    // Process the comma separated list or paramTypes
-                    // into an array of String class names
-                    ArrayList paramTypes = new ArrayList();
-                    StringTokenizer tokens = new StringTokenizer(paramTypesAttr, " \t\n\r,");
-                    while (tokens.hasMoreTokens()) {
-                            paramTypes.add(tokens.nextToken());
-                    }
-                    callMethodRule = new CallMethodRule( methodName,
-                                                        paramCount,
-                                                        (String[])paramTypes.toArray(new String[0]));
+                    String[] paramTypes = getParamTypes(paramTypesAttr);
+                    callMethodRule = new CallMethodRule(
+                        targetOffset, methodName, paramCount, paramTypes);
                 }
             }
             return callMethodRule;
         }
+
+        /**
+         * Process the comma separated list of paramTypes
+         * into an array of String class names
+         */
+        private String[] getParamTypes(String paramTypes) {
+            String[] paramTypesArray;
+            if( paramTypes != null ) {
+                ArrayList paramTypesList = new ArrayList();
+                StringTokenizer tokens = new StringTokenizer(
+                        paramTypes, " \t\n\r,");
+                while (tokens.hasMoreTokens()) {
+                    paramTypesList.add(tokens.nextToken());
+                }
+                paramTypesArray = (String[])paramTypesList.toArray(new String[0]);
+            } else {
+                paramTypesArray = new String[0];
+            }
+            return paramTypesArray;
+        }
     }
     
     /**
@@ -570,32 +606,96 @@ public class DigesterRuleParser extends RuleSetBase {
             int paramIndex = Integer.parseInt(attributes.getValue("paramnumber"));
             String attributeName = attributes.getValue("attrname");
             String fromStack = attributes.getValue("from-stack");
+            String stackIndex = attributes.getValue("stack-index");
             Rule callParamRule = null;
-            if (attributeName == null) {
-                if (fromStack == null) {
-                
-                    callParamRule = new CallParamRule( paramIndex );
-                
-                } else {
 
-                    callParamRule = new CallParamRule( paramIndex, Boolean.valueOf(fromStack).booleanValue());
-                    
+            if (attributeName == null) {
+                if (stackIndex != null) {                    
+                    callParamRule = new CallParamRule(
+                        paramIndex, Integer.parseInt(stackIndex));                
+                } else if (fromStack != null) {                
+                    callParamRule = new CallParamRule(
+                        paramIndex, Boolean.valueOf(fromStack).booleanValue());                
+                } else {
+                    callParamRule = new CallParamRule(paramIndex);     
                 }
             } else {
                 if (fromStack == null) {
-                    
-                    callParamRule = new CallParamRule( paramIndex, attributeName );
-                    
-                    
+                    callParamRule = new CallParamRule(paramIndex, attributeName);                    
                 } else {
                     // specifying both from-stack and attribute name is not allowed
-                    throw new RuntimeException("Attributes from-stack and attrname cannot both be present.");
+                    throw new RuntimeException(
+                        "Attributes from-stack and attrname cannot both be present.");
                 }
             }
             return callParamRule;
         }
     }
     
+    /**
+     * Factory for creating a ObjectParamRule
+     */
+    protected class ObjectParamRuleFactory extends AbstractObjectCreationFactory {
+        public Object createObject(Attributes attributes) throws Exception {
+            // create callparamrule
+            int paramIndex = Integer.parseInt(attributes.getValue("paramnumber"));
+            String attributeName = attributes.getValue("attrname");
+            String type = attributes.getValue("type");
+            String value = attributes.getValue("value");
+
+            Rule objectParamRule = null;
+
+            // type name is requried
+            if (type == null) {
+                throw new RuntimeException("Attribute 'type' is required.");
+            }
+
+            // create object instance
+            Object param = null;
+            Class clazz = Class.forName(type);
+            if (value == null) {
+                param = clazz.newInstance();
+            } else {
+                param = ConvertUtils.convert(value, clazz);
+            }
+
+            if (attributeName == null) {
+                objectParamRule = new ObjectParamRule(paramIndex, param);
+            } else {
+                objectParamRule = new ObjectParamRule(paramIndex, attributeName, param);
+            }
+            return objectParamRule;
+        }
+     }
+    
+        /**
+         * Factory for creating a NodeCreateRule
+         */
+    protected class NodeCreateRuleFactory extends AbstractObjectCreationFactory {
+
+        public Object createObject(Attributes attributes) throws Exception {
+
+            String nodeType = attributes.getValue("type");
+            if (nodeType == null || "".equals(nodeType)) {
+
+                // uses Node.ELEMENT_NODE
+                return new NodeCreateRule();
+            } else if ("element".equals(nodeType)) {
+
+                return new NodeCreateRule(Node.ELEMENT_NODE);
+            } else if ("fragment".equals(nodeType)) {
+
+                return new NodeCreateRule(Node.DOCUMENT_FRAGMENT_NODE);
+            } else {
+
+                throw new RuntimeException(
+                        "Unrecognized node type: "
+                                + nodeType
+                                + ".  This attribute is optional or can have a value of element|fragment.");
+            }
+        }
+    }    
+    
     /**
      * Factory for creating a FactoryCreateRule
      */
@@ -606,8 +706,7 @@ public class DigesterRuleParser extends RuleSetBase {
             boolean ignoreExceptions = 
                 "true".equalsIgnoreCase(attributes.getValue("ignore-exceptions"));
             return (attrName == null || attrName.length() == 0) ?
-                new FactoryCreateRule( className, ignoreExceptions)
-                :
+                new FactoryCreateRule( className, ignoreExceptions) :
                 new FactoryCreateRule( className, attrName, ignoreExceptions);
         }
     }
@@ -620,8 +719,7 @@ public class DigesterRuleParser extends RuleSetBase {
             String className = attributes.getValue("classname");
             String attrName = attributes.getValue("attrname");
             return (attrName == null || attrName.length() == 0) ?
-                new ObjectCreateRule( className)
-                :
+                new ObjectCreateRule( className) :
                 new ObjectCreateRule( className, attrName);
         }
     }
@@ -646,6 +744,19 @@ public class DigesterRuleParser extends RuleSetBase {
         }
     }
     
+    /**
+     * Factory for creating a SetNestedPropertiesRule
+     */
+    protected class SetNestedPropertiesRuleFactory extends AbstractObjectCreationFactory {
+        public Object createObject(Attributes attributes) {
+           boolean allowUnknownChildElements = 
+                "true".equalsIgnoreCase(attributes.getValue("allow-unknown-child-elements"));
+                SetNestedPropertiesRule snpr = new SetNestedPropertiesRule();
+                snpr.setAllowUnknownChildElements( allowUnknownChildElements );
+                return snpr;
+        }
+    }
+    
     /**
      * Factory for creating a SetTopRuleFactory
      */
@@ -654,8 +765,7 @@ public class DigesterRuleParser extends RuleSetBase {
             String methodName = attributes.getValue("methodname");
             String paramType = attributes.getValue("paramtype");
             return (paramType == null || paramType.length() == 0) ?
-                new SetTopRule( methodName)
-                :
+                new SetTopRule( methodName) :
                 new SetTopRule( methodName, paramType);
         }
     }
@@ -668,12 +778,24 @@ public class DigesterRuleParser extends RuleSetBase {
             String methodName = attributes.getValue("methodname");
             String paramType = attributes.getValue("paramtype");
             return (paramType == null || paramType.length() == 0) ?
-                new SetNextRule( methodName)
-                :
+                new SetNextRule( methodName) :
                 new SetNextRule( methodName, paramType);
         }
     }
     
+    /**
+     * Factory for creating a SetRootRuleFactory
+     */
+    protected class SetRootRuleFactory extends AbstractObjectCreationFactory {
+        public Object createObject(Attributes attributes) {
+            String methodName = attributes.getValue("methodname");
+            String paramType = attributes.getValue("paramtype");
+            return (paramType == null || paramType.length() == 0) ?
+                new SetRootRule( methodName) :
+                new SetRootRule( methodName, paramType);
+        }
+    }
+    
     /**
      * A rule for adding a attribute-property alias to the custom alias mappings of
      * the containing SetPropertiesRule rule.
@@ -681,9 +803,7 @@ public class DigesterRuleParser extends RuleSetBase {
     protected class SetPropertiesAliasRule extends Rule {
         
         /**
-         * <p>Base constructor.
-         *
-         * @param digester the Digester used to parse the rules XML file
+         * <p>Base constructor.</p>
          */
         public SetPropertiesAliasRule() {
             super();
@@ -701,4 +821,31 @@ public class DigesterRuleParser extends RuleSetBase {
             rule.addAlias(attrName, propName);
         }
     }
+
+    /**
+     * A rule for adding a attribute-property alias to the custom alias mappings of
+     * the containing SetNestedPropertiesRule rule.
+     */
+    protected class SetNestedPropertiesAliasRule extends Rule {
+        
+        /**
+         * <p>Base constructor.</p>
+         */
+        public SetNestedPropertiesAliasRule() {
+            super();
+        }
+        
+        /**
+         * Add the alias to the SetNestedPropertiesRule object created by the
+         * enclosing <set-nested-properties-rule> tag.
+         */
+        public void begin(Attributes attributes) {
+            String attrName = attributes.getValue("attr-name");
+            String propName = attributes.getValue("prop-name");
+    
+            SetNestedPropertiesRule rule = (SetNestedPropertiesRule) digester.peek();
+            rule.addAlias(attrName, propName);
+        }
+    }
+        
 }
