@@ -1,59 +1,20 @@
-/*
- * ====================================================================
+/* $Id: CallMethodRuleTestCase.java 472840 2006-11-09 10:18:34Z skitching $
  *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester;
@@ -146,8 +107,7 @@ public class CallMethodRuleTestCase extends TestCase {
 
     /**
      * Test method calls with the CallMethodRule rule. It should be possible
-     * to call any accessible method of the object on the top of the stack,
-     * even methods with no arguments.
+     * to call a method with no arguments using several rule syntaxes.
      */
     public void testBasic() throws SAXException, IOException {
         
@@ -160,11 +120,10 @@ public class CallMethodRuleTestCase extends TestCase {
         digester.addCallMethod("employee", "toString", 0, new String[] {});
         digester.addCallMethod("employee", "toString");
 
-        // Parse our test input
-        Object root1 = null;
-        // an exception will be thrown if the method can't be found
-        root1 = digester.parse(getInputStream("Test5.xml"));
-
+        // Parse our test input.
+        // An exception will be thrown if the method can't be found
+        Object root1 = digester.parse(getInputStream("Test5.xml"));
+        assertNotNull(root1);
     }
 
 
@@ -191,9 +150,9 @@ public class CallMethodRuleTestCase extends TestCase {
 
 
     /**
-     * Test method calls with the CallMethodRule rule. It should be possible
-     * to call any accessible method of the object on the top of the stack,
-     * even methods with no arguments.
+     * Test CallMethodRule variants which specify the classes of the
+     * parameters to target methods. String, int, boolean, float should all 
+     * be acceptable as parameter types.
      */
     public void testSettingProperties() throws SAXException, IOException {
             
@@ -297,9 +256,59 @@ public class CallMethodRuleTestCase extends TestCase {
 
 
     /**
-     * Test nested CallMethod rules.
+     * Test that the target object for a CallMethodRule is the object that was
+     * on top of the object stack when the CallMethodRule fired, even when other
+     * rules fire between the CallMethodRule and its associated CallParamRules.
+     * <p>
+     * The current implementation of CallMethodRule ensures this works by
+     * firing only at the end of the tag that CallMethodRule triggered on.
      */
-    public void testOrderNested() throws Exception {
+    public void testOrderNestedPartA() throws Exception {
+
+        // Configure the digester as required
+
+        // Here, we use the "grandchild element name" as a parameter to
+        // the created element, to ensure that all the params aren't
+        // avaiable to the CallMethodRule until some other rules have fired,
+        // in particular an ObjectCreateRule. The CallMethodRule should still
+        // function correctly in this scenario.
+        digester.addObjectCreate("toplevel/element", NamedBean.class);
+        digester.addCallMethod("toplevel/element", "setName", 1);
+        digester.addCallParam("toplevel/element/element/element", 0, "name");
+        
+        digester.addObjectCreate("toplevel/element/element", NamedBean.class);
+
+        // Parse our test input
+        NamedBean root1 = null;
+        try {
+            // an exception will be thrown if the method can't be found
+            root1 = (NamedBean) digester.parse(getInputStream("Test8.xml"));
+            
+        } catch (Throwable t) {
+            // this means that the method can't be found and so the test fails
+            fail("Digester threw Exception:  " + t);
+        }
+        
+        // if the CallMethodRule were to incorrectly invoke the method call
+        // on the second-created NamedBean instance, then the root one would
+        // have a null name. If it works correctly, the target element will
+        // be the first-created (root) one, despite the fact that a second
+        // object instance was created between the firing of the 
+        // CallMethodRule and its associated CallParamRule.
+        assertEquals("Wrong method call order", "C", root1.getName());
+    }
+
+    /**
+     * Test nested CallMethod rules.
+     * <p>
+     * The current implementation of CallMethodRule, in which the method is
+     * invoked in its end() method, causes behaviour which some users find
+     * non-intuitive. In this test it can be seen to "reverse" the order of
+     * data processed. However this is the way CallMethodRule has always 
+     * behaved, and it is expected that apps out there rely on this call order 
+     * so this test is present to ensure that no-one changes this behaviour.
+     */
+    public void testOrderNestedPartB() throws Exception {
         
         // Configure the digester as required
         StringBuffer word = new StringBuffer();
@@ -312,17 +321,15 @@ public class CallMethodRuleTestCase extends TestCase {
         try {
             // an exception will be thrown if the method can't be found
             root1 = digester.parse(getInputStream("Test8.xml"));
-            
+            assertNotNull(root1);
         } catch (Throwable t) {
             // this means that the method can't be found and so the test fails
             fail("Digester threw Exception:  " + t);
         }
         
-        assertEquals("Wrong method call order", "ABA", word.toString());
-
+        assertEquals("Wrong method call order", "CBA", word.toString());
     }
 
-
     public void testPrimitiveReading() throws Exception {
         StringReader reader = new StringReader(
             "<?xml version='1.0' ?><root><bean good='true'/><bean good='false'/><bean/>"
@@ -513,8 +520,6 @@ public class CallMethodRuleTestCase extends TestCase {
         digester.push(list);
         digester.parse(reader);
         
-        System.out.println(list);
-        
         NamedBean bean = (NamedBean) list.get(0);
         assertEquals("Wrong name (1)", "Simple", bean.getName());
         // these are added in deepest first order by the addRootRule
@@ -527,6 +532,151 @@ public class CallMethodRuleTestCase extends TestCase {
         bean = (NamedBean) list.get(1);
         assertEquals("Wrong name (5)", "Deepest", bean.getName());
     }
+    
+    public void testProcessingHook() throws Exception {
+        
+        class TestCallMethodRule extends CallMethodRule {
+            Object result;
+            TestCallMethodRule(String methodName, int paramCount)
+            {
+                super(methodName, paramCount);
+            }
+            protected void processMethodCallResult(Object result) {
+                this.result = result;
+            }
+        }
+    
+        StringReader reader = new StringReader(
+            "<?xml version='1.0' ?><root>"
+            + "<param class='float' coolness='false'>90</param></root>");
+        
+            
+        Digester digester = new Digester();
+        //SimpleLog log = new SimpleLog("{testTwoCalls:Digester]");
+        //log.setLevel(SimpleLog.LOG_LEVEL_TRACE);
+        //digester.setLogger(log);
+        
+        digester.addObjectCreate( "root/param", ParamBean.class );
+        digester.addSetNext( "root/param", "add" );
+        TestCallMethodRule rule = new TestCallMethodRule( "setThisAndThat" , 2 );
+        digester.addRule( "root/param", rule );
+        digester.addCallParam( "root/param", 0, "class" );
+        digester.addCallParam( "root/param", 1, "coolness" );
+        
+        ArrayList list = new ArrayList();
+        digester.push(list);
+        digester.parse(reader);
+    
+        assertEquals("Wrong number of objects created", 1, list.size());
+        assertEquals("Result not passed into hook", "The Other", rule.result);
+    }
+
+    /** Test for the PathCallParamRule */
+    public void testPathCallParam() throws Exception {
+        String xml = "<?xml version='1.0'?><main>"
+            + "<alpha><beta>Ignore this</beta></alpha>"
+            + "<beta><epsilon><gamma>Ignore that</gamma></epsilon></beta>"
+            + "</main>";
+    
+        SimpleTestBean bean = new SimpleTestBean();
+        bean.setAlphaBeta("[UNSET]", "[UNSET]");
+        
+        StringReader in = new StringReader(xml);
+        Digester digester = new Digester();
+        digester.setRules(new ExtendedBaseRules());
+        digester.addCallParamPath("*/alpha/?", 0);
+        digester.addCallParamPath("*/epsilon/?", 1);
+        digester.addCallMethod("main", "setAlphaBeta", 2);
+        
+        digester.push(bean);
+        
+        digester.parse(in);
+        
+        assertEquals("Test alpha property setting", "main/alpha/beta" , bean.getAlpha());
+        assertEquals("Test beta property setting", "main/beta/epsilon/gamma" , bean.getBeta());
+    }
+
+
+    /** 
+     * Test invoking an object which does not exist on the stack.
+     */
+    public void testCallInvalidTarget() throws Exception {
+    
+        Digester digester = new Digester();
+        digester.addObjectCreate("employee", HashMap.class);
+
+        // there should be only one object on the stack (index zero),
+        // so selecting a target object with index 1 on the object stack
+        // should result in an exception.
+        CallMethodRule r = new CallMethodRule(1, "put", 0);
+        digester.addRule("employee", r);
+        
+        try {
+            digester.parse(getInputStream("Test5.xml"));
+            fail("Exception should be thrown for invalid target offset");
+        }
+        catch(SAXException e) {
+            // ok, exception expected
+        }
+    }
+
+    /** 
+     * Test invoking an object which is at top-1 on the stack, like
+     * SetNextRule does...
+     */
+    public void testCallNext() throws Exception {
+    
+        Digester digester = new Digester();
+        digester.addObjectCreate("employee", HashMap.class);
+
+        digester.addObjectCreate("employee/address", Address.class);
+        digester.addSetNestedProperties("employee/address");
+        CallMethodRule r = new CallMethodRule(1, "put", 2);
+        digester.addRule("employee/address", r);
+        digester.addCallParam("employee/address/type", 0);
+        digester.addCallParam("employee/address", 1, 0);
+        
+        HashMap map = (HashMap) digester.parse(getInputStream("Test5.xml"));
+        
+        assertNotNull(map);
+        java.util.Set keys = map.keySet();
+        assertEquals(2, keys.size());
+        Address home = (Address) map.get("home");
+        assertNotNull(home);
+        assertEquals("HmZip", home.getZipCode());
+        Address office = (Address) map.get("office");
+        assertNotNull(office);
+        assertEquals("OfZip", office.getZipCode());
+    }
+
+    /** 
+     * Test invoking an object which is at the root of the stack, like
+     * SetRoot does...
+     */
+    public void testCallRoot() throws Exception {
+    
+        Digester digester = new Digester();
+        digester.addObjectCreate("employee", HashMap.class);
+
+        digester.addObjectCreate("employee/address", Address.class);
+        digester.addSetNestedProperties("employee/address");
+        CallMethodRule r = new CallMethodRule(-1, "put", 2);
+        digester.addRule("employee/address", r);
+        digester.addCallParam("employee/address/type", 0);
+        digester.addCallParam("employee/address", 1, 0);
+        
+        HashMap map = (HashMap) digester.parse(getInputStream("Test5.xml"));
+        
+        assertNotNull(map);
+        java.util.Set keys = map.keySet();
+        assertEquals(2, keys.size());
+        Address home = (Address) map.get("home");
+        assertNotNull(home);
+        assertEquals("HmZip", home.getZipCode());
+        Address office = (Address) map.get("office");
+        assertNotNull(office);
+        assertEquals("OfZip", office.getZipCode());
+    }
 
     // ------------------------------------------------ Utility Support Methods
 
