@@ -1,4 +1,4 @@
-/* $Id: DigesterTestCase.java 479207 2006-11-25 21:53:07Z craigmcc $
+/* $Id: DigesterTestCase.java 728879 2008-12-23 05:44:07Z rahul $
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -19,6 +19,9 @@
 
 package org.apache.commons.digester;
 
+import java.io.File;
+import java.io.InputStream;
+import java.io.Reader;
 import java.io.StringReader;
 import java.math.BigDecimal;
 import java.net.URL;
@@ -43,7 +46,7 @@ import org.xml.sax.helpers.AttributesImpl;
  * </p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 479207 $ $Date: 2006-11-25 16:53:07 -0500 (Sat, 25 Nov 2006) $
+ * @version $Revision: 728879 $ $Date: 2008-12-23 00:44:07 -0500 (Tue, 23 Dec 2008) $
  */
 
 public class DigesterTestCase extends TestCase {
@@ -124,6 +127,77 @@ public class DigesterTestCase extends TestCase {
     // ------------------------------------------------ Individual Test Methods
 
 
+    /**
+     * Test <code>null</code> parsing.
+     * (should lead to <code>IllegalArgumentException</code>s)
+     */
+    public void testNullFileParse() throws Exception {
+
+        try {
+            digester.parse((File) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+    public void testNullInputSourceParse() throws Exception {
+
+        try {
+            digester.parse((InputSource) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+    public void testNullInputStreamParse() throws Exception {
+
+        try {
+            digester.parse((InputStream) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+    public void testNullReaderParse() throws Exception {
+
+        try {
+            digester.parse((Reader) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+    public void testNullStringParse() throws Exception {
+
+        try {
+            digester.parse((String) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+    public void testNullURLParse() throws Exception {
+
+        try {
+            digester.parse((URL) null);
+            fail("Expected IllegalArgumentException with null argument");
+        } catch (IllegalArgumentException e) {
+            // expected
+        }
+
+    }
+
+
     /**
      * Test the basic property getters and setters.
      */
