@@ -1,63 +1,20 @@
-/*
- * $Header: /home/cvspublic/jakarta-commons/digester/src/test/org/apache/commons/digester/EBRTestCase.java,v 1.7 2003/03/12 22:10:51 rdonkin Exp $
- * $Revision: 1.7 $
- * $Date: 2003/03/12 22:10:51 $
+/* $Id: EBRTestCase.java 472840 2006-11-09 10:18:34Z skitching $
  *
- * ====================================================================
- *
- * The Apache Software License, Version 1.1
- *
- * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
- * reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowlegement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowlegement may appear in the software itself,
- *    if and wherever such third-party acknowlegements normally appear.
- *
- * 4. The names "The Jakarta Project", "Commons", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
- *
- */
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ * 
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */ 
 
 
 package org.apache.commons.digester;
@@ -74,7 +31,7 @@ import junit.framework.TestSuite;
  * <p> Runs standard tests for RulesBase as well as tests of extensions.
  *
  * @author Robert Burrell Donkin <robertdonkin@mac.com>
- * @version $Revision: 1.7 $ $Date: 2003/03/12 22:10:51 $
+ * @version $Revision: 472840 $ $Date: 2006-11-09 05:18:34 -0500 (Thu, 09 Nov 2006) $
  */
 
 
@@ -355,7 +312,6 @@ public class EBRTestCase extends RulesBaseTestCase {
 
 
         List list = null;
-        Iterator it = null;
 
         list = digester.getRules().match(null, "a");
 
@@ -386,7 +342,6 @@ public class EBRTestCase extends RulesBaseTestCase {
     }
     
     public void testAncesterMatch() throws Exception {
-        System.out.println("Starting ancester match...");
         // test fixed root ancester
         digester.getRules().clear();
         
@@ -445,6 +400,49 @@ public class EBRTestCase extends RulesBaseTestCase {
         list = digester.getRules().match(null, "b/b/c/e/d"); 
         assertEquals("Wild ancester match (2)", 1, list.size());
         assertEquals("Match missed (5)", "star-b-c-star" , ((TestRule) list.get(0)).getIdentifier());    
-        System.out.println("Finished ancester match.");
+    }
+    
+    public void testLongMatch() {
+        
+        digester.getRules().clear();
+
+        digester.addRule("a/b/c/d/*", new TestRule("a-b-c-d-star"));
+        
+        List list = digester.getRules().match(null, "a/b/c/d/e"); 
+        assertEquals("Long match (1)", 1, list.size());
+        assertEquals("Match missed (1)", "a-b-c-d-star" , ((TestRule) list.get(0)).getIdentifier()); 
+        
+        list = digester.getRules().match(null, "a/b/c/d/e/f");
+        assertEquals("Long match (2)", 1, list.size());
+        assertEquals("Match missed (2)", "a-b-c-d-star" , ((TestRule) list.get(0)).getIdentifier()); 
+        
+        list = digester.getRules().match(null, "a/b/c/d/e/f/g");
+        assertEquals("Long match (3)", 1, list.size());
+        assertEquals("Match missed (3)", "a-b-c-d-star" , ((TestRule) list.get(0)).getIdentifier()); 
+        
+        list = digester.getRules().match(null, "a/b/c/d");
+        assertEquals("Long match (4)", 0, list.size());
+    }
+    
+    public void testInstructors() {
+        digester.getRules().clear();
+        
+        digester.addRule("!instructors/*", new TestRule("instructors"));
+        digester.addRule("!instructor/*", new TestRule("instructor"));
+        
+        List list = digester.getRules().match(null, "instructors");
+        assertEquals("Only expect to match instructors", 1, list.size());
+        assertEquals("Instructors expected", "instructors" , ((TestRule) list.get(0)).getIdentifier()); 
+
+    }
+    
+    public void testMiddleInstructors() {
+        digester.getRules().clear();
+        
+        digester.addRule("!instructors/*", new TestRule("instructors"));
+        
+        List list = digester.getRules().match(null, "/tosh/instructors/fiddlesticks");
+        assertEquals("No matches expected", 0, list.size());
+
     }
 }
