@@ -1,4 +1,4 @@
-/* $Id: NodeCreateRuleTestCase.java 476030 2006-11-17 02:40:25Z rahul $
+/* $Id: NodeCreateRuleTestCase.java 730332 2008-12-31 07:09:35Z rahul $
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -45,7 +45,7 @@ import org.xml.sax.SAXException;
  * <p>Test case for the <code>NodeCreateRule</code>.
  * 
  * @author Christopher Lenz
- * @version $Revision: 476030 $ $Date: 2006-11-16 21:40:25 -0500 (Thu, 16 Nov 2006) $
+ * @version $Revision: 730332 $ $Date: 2008-12-31 02:09:35 -0500 (Wed, 31 Dec 2008) $
  */
 
 public class NodeCreateRuleTestCase extends TestCase {
@@ -120,7 +120,7 @@ public class NodeCreateRuleTestCase extends TestCase {
 
 
     /**
-     * Tests simple element construction, using the {@link TEST_XML TEST_XML}
+     * Tests simple element construction, using the {@link #TEST_XML}
      * XML input data.
      */
     public void testInvalidNodeTypes()
@@ -204,7 +204,7 @@ public class NodeCreateRuleTestCase extends TestCase {
     }
 
     /**
-     * Tests simple element construction, using the {@link TEST_XML TEST_XML}
+     * Tests simple element construction, using the {@link #TEST_XML}
      * XML input data.
      */
     public void testElement()
@@ -226,7 +226,7 @@ public class NodeCreateRuleTestCase extends TestCase {
 
 
     /**
-     * Tests simple fragment construction, using the {@link TEST_XML TEST_XML}
+     * Tests simple fragment construction, using the {@link #TEST_XML}
      * XML input data.
      */
     public void testDocumentFragment()
@@ -367,7 +367,7 @@ public class NodeCreateRuleTestCase extends TestCase {
         assertEquals(Node.ELEMENT_NODE, fragment.getFirstChild().getNodeType());
         Element address1 = (Element)fragment.getFirstChild();
         assertEquals("address", address1.getNodeName());
-        assertEquals("http://jakarta.apache.org/digester/Bar",
+        assertEquals("http://commons.apache.org/digester/Bar",
                      address1.getNamespaceURI());
         assertEquals("address", address1.getLocalName());
         assertEquals(5, address1.getAttributes().getLength());
@@ -380,7 +380,7 @@ public class NodeCreateRuleTestCase extends TestCase {
         assertEquals(Node.ELEMENT_NODE, fragment.getLastChild().getNodeType());
         Element address2 = (Element)fragment.getLastChild();
         assertEquals("address", address2.getNodeName());
-        assertEquals("http://jakarta.apache.org/digester/Bar",
+        assertEquals("http://commons.apache.org/digester/Bar",
                      address2.getNamespaceURI());
         assertEquals("address", address2.getLocalName());
         assertEquals(5, address2.getAttributes().getLength());
@@ -409,11 +409,11 @@ public class NodeCreateRuleTestCase extends TestCase {
         assertTrue(result instanceof Element);
         Element element = (Element)result;
         
-        assertNotNull(element.getAttributeNodeNS("http://jakarta.apache.org/digester/Bar", "test"));
-        assertEquals("MyTestAttribute", element.getAttributeNodeNS("http://jakarta.apache.org/digester/Bar", "test").getNodeValue());
-        assertEquals("test", element.getAttributeNodeNS("http://jakarta.apache.org/digester/Bar", "test").getLocalName());
-        assertEquals("bar", element.getAttributeNodeNS("http://jakarta.apache.org/digester/Bar", "test").getPrefix());
-        assertEquals("bar:test", element.getAttributeNodeNS("http://jakarta.apache.org/digester/Bar", "test").getName());
+        assertNotNull(element.getAttributeNodeNS("http://commons.apache.org/digester/Bar", "test"));
+        assertEquals("MyTestAttribute", element.getAttributeNodeNS("http://commons.apache.org/digester/Bar", "test").getNodeValue());
+        assertEquals("test", element.getAttributeNodeNS("http://commons.apache.org/digester/Bar", "test").getLocalName());
+        assertEquals("bar", element.getAttributeNodeNS("http://commons.apache.org/digester/Bar", "test").getPrefix());
+        assertEquals("bar:test", element.getAttributeNodeNS("http://commons.apache.org/digester/Bar", "test").getName());
 
     }      
     
@@ -463,6 +463,28 @@ public class NodeCreateRuleTestCase extends TestCase {
 
     }
 
+    /**
+     * This unit test checks that text nodes are correctly created when
+     * xml entities are used. In particular, this usually causes the xml
+     * parser to make multiple invocations of the characters(..) sax
+     * callback, rather than just one.
+     */
+    public void testEntityText() throws Exception {
+        String TEST_XML2 =
+            "<?xml version='1.0'?><root><alpha>&#65; &#65;</alpha></root>";
+
+        digester.addRule("root/alpha", new NodeCreateRule());
+        Object result = digester.parse(new StringReader(TEST_XML2));
+
+        assertNotNull(result);
+        assertTrue(result instanceof Element);
+        Element element = (Element)result;
+        assertEquals("alpha", element.getNodeName());
+        assertNull(((Element)element).getLocalName());
+        assertNull(((Element)element).getNamespaceURI());
+        assertEquals(1, element.getChildNodes().getLength());
+        assertEquals("A A", element.getFirstChild().getNodeValue());
+    }
     // ------------------------------------------------ Utility Support Methods
 
 
