@@ -1,4 +1,4 @@
-/* $Id: RuleTestCase.java 471661 2006-11-06 08:09:25Z skitching $
+/* $Id: RuleTestCase.java 730332 2008-12-31 07:09:35Z rahul $
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -36,7 +36,7 @@ import org.xml.sax.SAXException;
  *
  * @author Craig R. McClanahan
  * @author Janek Bogucki
- * @version $Revision: 471661 $ $Date: 2006-11-06 03:09:25 -0500 (Mon, 06 Nov 2006) $
+ * @version $Revision: 730332 $ $Date: 2008-12-31 02:09:35 -0500 (Wed, 31 Dec 2008) $
  */
 
 public class RuleTestCase extends TestCase {
@@ -345,7 +345,7 @@ public class RuleTestCase extends TestCase {
         // Configure the digester as required
         digester.setNamespaceAware(true);
         RuleSet rs = new TestRuleSet(null,
-                "http://jakarta.apache.org/digester/Foo");
+                "http://commons.apache.org/digester/Foo");
         digester.addRuleSet(rs);
 
         // Parse our test input.
@@ -380,7 +380,7 @@ public class RuleTestCase extends TestCase {
         // Configure the digester as required
         digester.setNamespaceAware(true);
         RuleSet rs = new TestRuleSet(null,
-                "http://jakarta.apache.org/digester/Foo");
+                "http://commons.apache.org/digester/Foo");
         digester.addRuleSet(rs);
 
         // Parse our test input.
@@ -644,7 +644,7 @@ public class RuleTestCase extends TestCase {
     /**
      * Validate the assertions for ObjectCreateRule3.
      *
-     * @param object Root object returned by <code>digester.parse()</code>
+     * @param root Root object returned by <code>digester.parse()</code>
      */
     protected void validateObjectCreate3(Object root) {
 
